--- /dev/null	Thu Oct  3 17:49:52 2019
+++ new/usr/src/lib/libc/port/gen/cfsetspeed.c	Thu Oct  3 17:49:52 2019
@@ -0,0 +1,113 @@
+/*
+ * CDDL HEADER START
+ *
+ * The contents of this file are subject to the terms of the
+ * Common Development and Distribution License (the "License").
+ * You may not use this file except in compliance with the License.
+ *
+ * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
+ * or http://www.opensolaris.org/os/licensing.
+ * See the License for the specific language governing permissions
+ * and limitations under the License.
+ *
+ * When distributing Covered Code, include this CDDL HEADER in each
+ * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
+ * If applicable, add the following below this CDDL HEADER, with the
+ * fields enclosed by brackets "[]" replaced with your own identifying
+ * information: Portions Copyright [yyyy] [name of copyright owner]
+ *
+ * CDDL HEADER END
+ */
+
+/*
+ * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
+ * Use is subject to license terms.
+ */
+
+/*      Copyright (c) 1988 AT&T */
+/*        All Rights Reserved   */
+
+/* Copyright 2019, Joyent, Inc. */
+
+#pragma weak _cfsetispeed = cfsetispeed
+#pragma weak _cfsetospeed = cfsetospeed
+#pragma weak _cfgetospeed = cfgetospeed
+#pragma weak _cfgetispeed = cfgetispeed
+
+#include <sys/types.h>
+#include <sys/termios.h>
+
+/*
+ * returns input baud rate stored in c_cflag pointed by termios_p
+ */
+
+speed_t
+cfgetispeed(const struct termios *termios_p)
+{
+	return (termios_p->c_cflag & CIBAUDEXT ?
+	    ((termios_p->c_cflag & CIBAUD) >> 16) + (CIBAUD >> 16) + 1 :
+	    (termios_p->c_cflag & CIBAUD) >> 16);
+}
+
+/*
+ * returns output baud rate stored in c_cflag pointed by termios_p
+ */
+
+speed_t
+cfgetospeed(const struct termios *termios_p)
+{
+	return (termios_p->c_cflag & CBAUDEXT ?
+	    (termios_p->c_cflag & CBAUD) + CBAUD + 1 :
+	    termios_p->c_cflag & CBAUD);
+}
+
+/*
+ * sets the input baud rate stored in c_cflag to speed
+ */
+
+int
+cfsetispeed(struct termios *termios_p, speed_t speed)
+{
+	/*
+	 * If the input speed is zero, set it to output speed
+	 */
+	if (speed == 0) {
+		speed = termios_p->c_cflag & CBAUD;
+		if (termios_p->c_cflag & CBAUDEXT)
+			speed += (CBAUD + 1);
+	}
+
+	if ((speed << 16) > CIBAUD) {
+		termios_p->c_cflag |= CIBAUDEXT;
+		speed -= ((CIBAUD >> 16) + 1);
+	} else
+		termios_p->c_cflag &= ~CIBAUDEXT;
+	termios_p->c_cflag =
+	    (termios_p->c_cflag & ~CIBAUD) | ((speed << 16) & CIBAUD);
+	return (0);
+}
+
+/*
+ * sets the output baud rate stored in c_cflag to speed
+ */
+
+int
+cfsetospeed(struct termios *termios_p, speed_t speed)
+{
+	if (speed > CBAUD) {
+		termios_p->c_cflag |= CBAUDEXT;
+		speed -= (CBAUD + 1);
+	} else
+		termios_p->c_cflag &= ~CBAUDEXT;
+
+	termios_p->c_cflag =
+	    (termios_p->c_cflag & ~CBAUD) | (speed & CBAUD);
+	return (0);
+}
+
+int
+cfsetspeed(struct termios *termios_p, speed_t speed)
+{
+	(void) cfsetispeed(termios_p, speed);
+	(void) cfsetospeed(termios_p, speed);
+}
